

/*
 * Copyright (c) 2006 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * -Redistribution of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 * -Redistribution in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *
 * Neither the name of Sun Microsystems, Inc. or the names of contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING
 * ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN MIDROSYSTEMS, INC. ("SUN")
 * AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE
 * AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR ANY LOST
 * REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY
 * OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
 * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * You acknowledge that this software is not designed, licensed or intended
 * for use in the design, construction, operation or maintenance of any
 * nuclear facility.
 */

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.ByteLookupTable;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.awt.image.LookupOp;
import java.awt.image.WritableRaster;
import java.io.File;
import java.io.IOException;
import java.util.TreeSet;

import javax.imageio.ImageIO;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

public class SaveImage extends Component implements ActionListener {

	String descs[] = { "Original", "Convolve : LowPass", "Convolve : Sharpen",
			"LookupOp", "Grayscale"};

	int opIndex;

	private BufferedImage bi, biFiltered;

	int w, h;

	public static final float[] SHARPEN3x3 = { // sharpening filter kernel
	0.f, -1.f, 0.f, -1.f, 5.f, -1.f, 0.f, -1.f, 0.f };

	public static final float[] BLUR3x3 = { 0.1f, 0.1f, 0.1f, // low-pass
			// filter kernel
			0.1f, 0.2f, 0.1f, 0.1f, 0.1f, 0.1f };

	public SaveImage(File imgFile) {
		try {
			bi = ImageIO.read(imgFile);
			w = bi.getWidth(null);
			h = bi.getHeight(null);
			if (bi.getType() != BufferedImage.TYPE_INT_RGB) {
				BufferedImage bi2 = new BufferedImage(w, h,
						BufferedImage.TYPE_INT_RGB);
				Graphics big = bi2.getGraphics();
				big.drawImage(bi, 0, 0, null);
				biFiltered = bi = bi2;
			}
		} catch (IOException e) {
			System.out.println("Image could not be read");
			System.exit(1);
		}
	}

	public Dimension getPreferredSize() {
		return new Dimension(w, h);
	}

	String[] getDescriptions() {
		return descs;
	}

	void setOpIndex(int i) {
		opIndex = i;
	}

	public void paint(Graphics g) {
		filterImage();
		g.drawImage(biFiltered, 0, 0, null);
	}

	int lastOp;

	public void filterImage() {
		BufferedImageOp op = null;

		if (opIndex == lastOp) {
			return;
		}
		lastOp = opIndex;
		switch (opIndex) {

		case 0:
			biFiltered = bi; /* original */
			return;
		case 1: /* low pass filter */
		case 2: /* sharpen */
			float[] data = (opIndex == 1) ? BLUR3x3 : SHARPEN3x3;
			op = new ConvolveOp(new Kernel(3, 3, data), ConvolveOp.EDGE_NO_OP,
					null);

			break;

		case 3: /* lookup */
			byte lut[] = new byte[256];
			for (int j = 0; j < 256; j++) {
				lut[j] = (byte) (256 - j);
			}
			ByteLookupTable blut = new ByteLookupTable(0, lut);
			op = new LookupOp(blut, null);
			break;
		case 4:
			grayscale();
			op = new ConvolveOp(new Kernel(1, 1, new float[]{1}), ConvolveOp.EDGE_NO_OP,
					null);
			break;
		}

		/*
		 * Rather than directly drawing the filtered image to the destination,
		 * filter it into a new image first, then that filtered image is ready
		 * for writing out or painting.
		 */
		biFiltered = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
		op.filter(bi, biFiltered);
	}

	private void grayscale() {
		WritableRaster raster = biFiltered.getRaster();
		for (int i = 0; i < biFiltered.getWidth(); i++) {
			for (int j = 0; j < biFiltered.getHeight(); j++) {
				int[] rgb = raster.getPixel(i, j, (int[]) null);
				int intensity = (rgb[0] + rgb[1] + rgb[2]) / 3;
				raster.setPixel(i, j, new int[] { intensity, intensity,
						intensity });
			}
		}
	}

	/* Return the formats sorted alphabetically and in lower case */
	public String[] getFormats() {
		String[] formats = ImageIO.getWriterFormatNames();
		TreeSet formatSet = new TreeSet();
		for (int i=0; i<formats.length; i++) {
			formatSet.add(formats[i].toLowerCase());
		}
                //return formatSet.toArray(new String[0]);
                Object[] objSet = formatSet.toArray();
                String[] sortedFormats = new String[objSet.length];
                for (int i=0; i<objSet.length; i++)
                  sortedFormats[i] = objSet[i].toString();
                return sortedFormats;
	}

	public void actionPerformed(ActionEvent e) {
		JComboBox cb = (JComboBox) e.getSource();
		if (cb.getActionCommand().equals("SetFilter")) {
			setOpIndex(cb.getSelectedIndex());
			repaint();
		} else if (cb.getActionCommand().equals("Formats")) {
			/*
			 * Save the filtered image in the selected format. The selected item
			 * will be the name of the format to use
			 */
			String format = (String) cb.getSelectedItem();
			/*
			 * Use the format name to initialise the file suffix. Format names
			 * typically correspond to suffixes
			 */
			File saveFile = new File("savedimage." + format);
			JFileChooser chooser = new JFileChooser();
			chooser.setSelectedFile(saveFile);
			int rval = chooser.showSaveDialog(cb);
			if (rval == JFileChooser.APPROVE_OPTION) {
				saveFile = chooser.getSelectedFile();
				/*
				 * Write the filtered image in the selected format, to the file
				 * chosen by the user.
				 */
				try {
					ImageIO.write(biFiltered, format, saveFile);
				} catch (IOException ex) {
				}
			}
		}
	};

	public static void main(String s[]) {
		JFrame f = new JFrame("Save Image Sample");
		f.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});

		JFileChooser jfc = new JFileChooser();
		int r = jfc.showOpenDialog(f);
		if (r == JFileChooser.APPROVE_OPTION) {
			SaveImage si = new SaveImage(jfc.getSelectedFile());
			f.add("Center", si);
			JComboBox choices = new JComboBox(si.getDescriptions());
			choices.setActionCommand("SetFilter");
			choices.addActionListener(si);
			JComboBox formats = new JComboBox(si.getFormats());
			formats.setActionCommand("Formats");
			formats.addActionListener(si);
			JPanel panel = new JPanel();
			panel.add(choices);
			panel.add(new JLabel("Save As"));
			panel.add(formats);
			f.add("South", panel);
			f.pack();
			f.setVisible(true);
		}

	}
}
